@@ -20,11 +20,7 @@
 /// # Example
 ///
 /// ```rust
-/// # #[macro_use] extern crate sudo_plugin;
-/// extern crate libc;
-///
-/// # fn main() { } // TODO(rust 1.27): remove this line
-///
+/// use sudo_plugin::*;
 /// use sudo_plugin::errors::*;
 /// use std::io::Write;
 ///
@@ -36,21 +32,24 @@
 /// }
 ///
 /// struct Example {
+///     plugin: &'static sudo_plugin::Plugin
 /// }
 ///
 /// impl Example {
 ///     fn open(plugin: &'static sudo_plugin::Plugin) -> Result<Self> {
 ///         plugin.stdout().write(b"example sudo plugin initialized");
 ///
-///         Ok(Example {})
+///         Ok(Example { plugin })
 ///     }
 ///
 ///     fn close(&mut self, _: i32, _: i32) {
-///         println!("example sudo plugin exited");
+///         self.plugin.stdout().write(b"example sudo plugin exited");
 ///     }
 ///
 ///     fn log_stdout(&mut self, _: &[u8]) -> Result<()> {
-///         println!("example sudo plugin received output on stdout");
+///         self.plugin.stdout().write(
+///             b"example sudo plugin received output on stdout"
+///         );
 ///
 ///         Ok(())
 ///     }
@@ -68,26 +67,28 @@
 #[macro_export]
 macro_rules! sudo_io_plugin {
     ( $name:ident : $ty:ty { $($cb:ident : $fn:ident),* $(,)* } ) => {
-        static mut PLUGIN:   Option<sudo_plugin::Plugin> = None;
-        static mut INSTANCE: Option<$ty>                 = None;
+        use ::sudo_plugin::errors::AsSudoPluginRetval;
+
+        static mut PLUGIN:   Option<::sudo_plugin::Plugin> = None;
+        static mut INSTANCE: Option<$ty>                   = None;
+
+        const VERSION: Option<&'static str> = option_env!("CARGO_PKG_VERSION");
 
         #[no_mangle]
         #[allow(non_upper_case_globals)]
         #[allow(missing_docs)]
-        pub static $name: sudo_plugin::sys::io_plugin = {
-            sudo_plugin::sys::io_plugin {
-                open: sudo_io_static_fn!(
-                    open, $name, PLUGIN, INSTANCE, $ty, open
-                ),
-
+        pub static $name: ::sudo_plugin::sys::io_plugin = {
+            ::sudo_plugin::sys::io_plugin {
+                // construct the plugin using any callbacks specified
                 $( $cb: sudo_io_fn!($cb, $name, PLUGIN, INSTANCE, $fn) ),*,
 
-                .. sudo_plugin::sys::io_plugin {
-                    type_:            sudo_plugin::sys::SUDO_IO_PLUGIN,
-                    version:          sudo_plugin::sys::SUDO_API_VERSION,
-                    open:             None,
+                // and for anything not specified, use the defaults
+                .. ::sudo_plugin::sys::io_plugin {
+                    type_:            ::sudo_plugin::sys::SUDO_IO_PLUGIN,
+                    version:          ::sudo_plugin::sys::SUDO_API_VERSION,
+                    open:             Some(open),
                     close:            None,
-                    show_version:     None,
+                    show_version:     Some(show_version),
                     log_ttyin:        None,
                     log_ttyout:       None,
                     log_stdin:        None,
@@ -99,23 +100,11 @@ macro_rules! sudo_io_plugin {
                 }
             }
         };
-    }
-}
 
-#[macro_export]
-macro_rules! sudo_io_static_fn {
-    (
-        open ,
-        $name:tt ,
-        $plugin:expr ,
-        $instance:expr ,
-        $ty:ty ,
-        $fn:ident
-    ) => {{
         unsafe extern "C" fn open(
             version:            ::libc::c_uint,
-            conversation:       sudo_plugin::sys::sudo_conv_t,
-            plugin_printf:      sudo_plugin::sys::sudo_printf_t,
+            conversation:       ::sudo_plugin::sys::sudo_conv_t,
+            plugin_printf:      ::sudo_plugin::sys::sudo_printf_t,
             settings_ptr:       *const *mut ::libc::c_char,
             user_info_ptr:      *const *mut ::libc::c_char,
             command_info_ptr:   *const *mut ::libc::c_char,
@@ -125,8 +114,8 @@ macro_rules! sudo_io_static_fn {
             plugin_options_ptr: *const *mut ::libc::c_char,
         ) -> ::libc::c_int {
             unsafe fn stderr(
-                printf: sudo_plugin::sys::sudo_printf_t,
-                error:  &Error,
+                printf: ::sudo_plugin::sys::sudo_printf_t,
+                error:  &::sudo_plugin::errors::Error,
             ) -> ::libc::c_int {
                 // if printf is a NULL pointer or if the `write_error`
                 // call fails, we don't really have anything productive
@@ -136,16 +125,16 @@ macro_rules! sudo_io_static_fn {
                         ::std::sync::Mutex::new(printf)
                     );
 
-                    let _ = sudo_plugin::Printf {
+                    let _ = ::sudo_plugin::Printf {
                         facility: printf,
-                        level:    sudo_plugin::sys::SUDO_CONV_ERROR_MSG,
+                        level:    ::sudo_plugin::sys::SUDO_CONV_ERROR_MSG,
                     }.write_error(stringify!($name).as_bytes(), error);
                 }
 
                 return error.as_sudo_io_plugin_open_retval();
             }
 
-            let plugin = sudo_plugin::Plugin::new(
+            let plugin = ::sudo_plugin::Plugin::new(
                 version,
                 argc, argv,
                 conversation,
@@ -158,32 +147,58 @@ macro_rules! sudo_io_static_fn {
             );
 
             match plugin {
-                Ok(p)  => $plugin = Some(p),
+                Ok(p)  => PLUGIN = Some(p),
                 Err(e) => return stderr(plugin_printf, &e),
             }
 
             // unwrap should be panic-safe here, since we just assigned
             // a value to $plugin
-            let instance = <$ty>::$fn($plugin.as_ref().unwrap());
+            let plugin = PLUGIN.as_ref().unwrap();
 
-            match instance {
-                Ok(i)  => $instance = Some(i),
-                Err(e) => return stderr(plugin_printf, &e),
+            // if the command is empty, to the best of my knowledge
+            // we're being called with `-V` to report our version; in
+            // this case there's no reason to fully the plugin through
+            // its `open` function
+            if plugin.command_info.command == ::std::path::PathBuf::default() {
+                return ::sudo_plugin::sys::SUDO_PLUGIN_OPEN_SUCCESS;
             }
 
-            sudo_plugin::sys::SUDO_PLUGIN_OPEN_SUCCESS
+            // call the plugin's `open` function
+            match <$ty>::open(plugin) {
+                Ok(i)  => INSTANCE = Some(i),
+                Err(e) => return stderr(plugin_printf, &e.into()),
+            }
+
+            ::sudo_plugin::sys::SUDO_PLUGIN_OPEN_SUCCESS
         }
 
-        Some(open)
-    }};
+        unsafe extern "C" fn show_version(
+            _verbose: ::libc::c_int,
+        ) -> ::libc::c_int {
+            if let Some(plugin) = PLUGIN.as_ref() {
+                // disable the write_literal lint since it has a known
+                // bug that fires when you use a macro that expands to
+                // a literal (e.g., `stringify!`)
+                #[cfg_attr(feature="cargo-clippy", allow(clippy::write_literal))]
+                let _ = writeln!(plugin.stdout(),
+                    "{} I/O plugin version {}",
+                    stringify!($name), VERSION.unwrap_or("unknown")
+                );
+            }
+
+            0
+        }
+    }
 }
 
+/// Internal macro used by `sudo_io_plugin` that  generates the actual
+/// callback implementations for I/O plugins.
 #[macro_export]
 macro_rules! sudo_io_fn {
     ( close , $name:tt , $plugin:expr , $instance:expr , $fn:ident ) => {{
         unsafe extern "C" fn close(
             exit_status: ::libc::c_int,
-            error:       ::libc::c_int
+            error:       ::libc::c_int,
         ) {
             if let Some(i) = $instance.as_mut() {
                 i.$fn(exit_status as _, error as _)
@@ -223,17 +238,19 @@ macro_rules! sudo_io_fn {
     ) => {{
         unsafe extern "C" fn $log_fn(
             buf: *const ::libc::c_char,
-            len: ::libc::c_uint,
+            len:        ::libc::c_uint,
         ) -> ::libc::c_int {
-            let slice = std::slice::from_raw_parts(
+            let slice = ::std::slice::from_raw_parts(
                 buf as *const _,
                 len as _,
             );
 
-            let result = $instance
+            let result : ::std::result::Result<(), ::sudo_plugin::errors::Error> = $instance
                 .as_mut()
-                .ok_or(::sudo_plugin::errors::ErrorKind::Uninitialized.into())
-                .and_then(|i| i.$fn(slice) );
+                .map_or_else(
+                  || Err(::sudo_plugin::errors::ErrorKind::Uninitialized.into()),
+                  |i| i.$fn(slice).map_err(|e| e.into()),
+                );
 
             // if there was an error (and we can unwrap the plugin),
             // write it out
@@ -255,10 +272,12 @@ macro_rules! sudo_io_fn {
             lines: ::libc::c_uint,
             cols:  ::libc::c_uint,
         ) {
-            let result = $instance
+            let result : ::std::result::Result<(), ::sudo_plugin::errors::Error> = $instance
                 .as_mut()
-                .ok_or(::sudo_plugin::errors::ErrorKind::Uninitialized.into())
-                .and_then(|i| i.$fn(lines as _, cols as _) );
+                .map_or_else(
+                  || Err(::sudo_plugin::errors::ErrorKind::Uninitialized.into()),
+                  |i| i.$fn(slice).map_err(|e| e.into()),
+                );
 
             // if there was an error (and we can unwrap the plugin),
             // write it out
