@@ -30,8 +30,6 @@ use self::command_info::CommandInfo;
 use self::settings::Settings;
 use self::user_info::UserInfo;
 
-use sudo_plugin_sys;
-
 use std::collections::HashSet;
 use std::path::PathBuf;
 use std::ffi::{CString, CStr};
@@ -90,8 +88,10 @@ impl Plugin {
     /// facilities.
     ///
     /// Returns an error if there was a problem initializing the plugin.
-    #[cfg_attr(feature="cargo-clippy", allow(cast_sign_loss))]
-    #[cfg_attr(feature="cargo-clippy", allow(too_many_arguments))]
+    #[cfg_attr(feature="cargo-clippy", allow(clippy::new_ret_no_self))]
+    #[cfg_attr(feature="cargo-clippy", allow(clippy::cast_sign_loss))]
+    #[cfg_attr(feature="cargo-clippy", allow(clippy::too_many_arguments))]
+    #[cfg_attr(feature="cargo-clippy", allow(clippy::missing_safety_doc))]
     pub unsafe fn new(
         version:        c_uint,
         argc:           c_int,
@@ -249,7 +249,7 @@ impl Plugin {
 pub struct Tty(File);
 
 impl Tty {
-    fn try_from(path: &Path) -> io::Result<Tty> {
+    fn try_from(path: &Path) -> io::Result<Self> {
         OpenOptions::new().write(true).open(path).map(Tty)
     }
 }
@@ -324,17 +324,17 @@ impl Write for Printf {
             // cast is only necessary because Rust interprets the
             // `#define`'d constants as `u32` when they're treated by
             // sudo as `i32`.
-            #[cfg_attr(feature="cargo-clippy", allow(cast_possible_wrap))]
+            #[cfg_attr(feature="cargo-clippy", allow(clippy::cast_possible_wrap))]
             (self.facility.lock().unwrap())(self.level as i32, message.as_ptr())
         };
 
         if ret == -1 {
-            Err(io::Error::last_os_error())?;
+            return Err(io::Error::last_os_error());
         }
 
         // TODO: replace the cast, but for now we've checked for it
         // being negative so there's no possibility of wraparound
-        #[cfg_attr(feature="cargo-clippy", allow(cast_sign_loss))]
+        #[cfg_attr(feature="cargo-clippy", allow(clippy::cast_sign_loss))]
         Ok(ret as _)
     }
 
