@@ -12,7 +12,7 @@
 // implied. See the License for the specific language governing
 // permissions and limitations under the License.
 
-use super::errors::*;
+use crate::errors::*;
 
 use std::fmt;
 
@@ -37,8 +37,8 @@ impl Version {
         &MINIMUM
     }
 
-    pub fn supported(&self) -> bool {
-        self >= Self::minimum()
+    pub fn supported(self) -> bool {
+        self >= *Self::minimum()
     }
 
     pub fn check(self) -> Result<Self> {
@@ -51,7 +51,7 @@ impl Version {
 }
 
 impl From<c_uint> for Version {
-    #[cfg_attr(feature="cargo-clippy", allow(cast_possible_truncation))]
+    #[cfg_attr(feature="cargo-clippy", allow(clippy::cast_possible_truncation))]
     fn from(version: c_uint) -> Self {
         Self {
             major: (version >> 16)     as _,
@@ -61,7 +61,7 @@ impl From<c_uint> for Version {
 }
 
 impl fmt::Display for Version {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "{}.{}", self.major, self.minor)
     }
 }
